import * as fs from 'fs';
import * as csvtojson from 'csvtojson';
import { BadRequestException, Injectable } from '@nestjs/common';
import { CarreraService } from '../carrera/carrera.service';
import { EquipoService } from '../equipo/equipo.service';
import { InstitucionService } from '../institucion/institucion.service';
import { TipoUsuarioService } from '../tipo-usuario/tipo-usuario.service';
import { UsuarioService } from '../usuario/usuario.service';
import { Carrera } from '../carrera/entity/carrera.entity';
import { Institucion } from '../institucion/entity/institucion.entity';
import { TipoUsuario } from '../tipo-usuario/entity/tipo-usuario.entity';
import { UploadFileCargaMasivaUsuarioDto } from './dto/upload-file-carga-masiva-usuario.dto';

@Injectable()
export class UploadFileService {
  constructor(
    private carreraService: CarreraService,
    private equipoService: EquipoService,
    private institucionService: InstitucionService,
    private tipoUsuarioService: TipoUsuarioService,
    private usuarioService: UsuarioService,
  ) {}

  async createEquipos(file: Express.Multer.File, id_institucion: number) {
    const path = `${file.destination}/${file.filename}`;
    const institucion = await this.institucionService.findById(id_institucion);

    if (!file) throw new BadRequestException('No se mando un archivo.');
    csvtojson()
      .fromFile(path)
      .then(async (equipos) => {
        for (let i = 0; i < equipos.length; i++) {}
      });
  }

  async createUsuarios(file: Express.Multer.File, id_institucion: number) {
    const path = `${file.destination}/${file.filename}`;
    const institucion = await this.institucionService.findById(id_institucion);
    const errores: string[] = [];

    // if (!file) throw new BadRequestException('No se mando un archivo.');
    return csvtojson()
      .fromFile(path)
      .then(async (usuarios: [UploadFileCargaMasivaUsuarioDto]) => {
        for (let i = 0; i < usuarios.length; i++) {
          let error = this.errorBase(i);

          if (
            !usuarios[i].numero_cuenta ||
            !usuarios[i].tipo_usuario ||
            !usuarios[i].nombres ||
            !usuarios[i].apellido_m ||
            !usuarios[i].apellido_p
          ) {
            if (!usuarios[i].numero_cuenta)
              error += ' falta del campo numero_cuenta';
            if (error != this.errorBase(i)) error += ',';
            if (!usuarios[i].tipo_usuario)
              error += ' falta del campo tipo_usuario';
            if (error != this.errorBase(i)) error += ',';
            if (!usuarios[i].nombres) error += ' falta del campo nombres';
            if (error != this.errorBase(i)) error += ',';
            if (!usuarios[i].apellido_m) error += ' falta del campo apellido_m';
            if (error != this.errorBase(i)) error += ',';
            if (!usuarios[i].apellido_p) error += ' falta del campo apellido_p';
            error += '.';
            errores.push(error);
            continue;
          }

          const tipoUsuario = await this.tipoUsuarioService.findByTipoUsuario(
            usuarios[i].tipo_usuario,
            false,
          );
          const usuario = await this.usuarioService.findByUsuario(
            usuarios[i].numero_cuenta,
            false,
          );
          const carrera =
            usuarios[i].carrera && tipoUsuario.id_tipo_usuario > 5
              ? await this.carreraService.findByCarrera(
                  usuarios[i].carrera,
                  false,
                )
              : null;

          console.log(usuario);
          if (!tipoUsuario || (tipoUsuario.id_tipo_usuario > 5 && !carrera)) {
            if (tipoUsuario.id_tipo_usuario !== 5 && !carrera)
              error += ' falta del campo carrera para el alumno';
            if (error != this.errorBase(i)) error += ',';
            if (!tipoUsuario) error += ' no existe este tipo de usuario';
            error += '.';
            errores.push(error);
            continue;
          }
          if (usuario) {
            const data: {
              carrera?: Carrera;
              id_usuario: number;
              institucion: Institucion;
              tipoUsuario: TipoUsuario;
            } = {
              id_usuario: usuario.id_usuario,
              institucion,
              tipoUsuario,
            };

            if (carrera) data.carrera = carrera;
            // await this.usuarioService.update(data).then((usuarioUpdated) => {});
          } else
            await this.usuarioService
              .create(
                usuarios[i].numero_cuenta,
                institucion.id_institucion,
                tipoUsuario.id_tipo_usuario,
                carrera ? carrera.id_carrera : null,
              )
              .then((usuario) => {});
        }
        return fs.unlink(path, (err) => {
          console.log(err);
        });
      })
      .then((res) => {
        console.log(res);
        return { message: 'Se subió y cargo correctamente tu archivo csv.' };
      });
  }

  downloadLogo(id_institucion: number) {}

  errorBase(index) {
    return `Se salto la linea ${index + 2} por:`;
  }

  uploadLogo(file: Express.Multer.File, id_institucion: number) {}
}
